/*****************************************************************************
 *                                                                           *
 *        **********                                                         *
 *       ************                                                        *
 *      ***        ***                                                       *
 *      ***   +++   ***                                                      *
 *      ***   + +   ***                                                      *
 *      ***   +                            CHIPCON CC1010                    *
 *      ***   + +   ***                 HAL - ReadPotSensor                  *
 *      ***   +++   ***                                                      *
 *      ***       ***                                                        *
 *       ***********                                                         *
 *        *********                                                          *
 *                                                                           *
 *****************************************************************************
 * Reads the CC1010EB potentiometer sensor.                                  *
 *****************************************************************************
 * Author:              TOS                                                  *
 *****************************************************************************
 * Revision history:                                                         *
 *                                                                           *
 * $Log: ReadPotSensor.c,v $
 * Revision 1.1  2004/01/08 19:15:32  tos
 * Initial version in CVS.
 *
 *
 *                                                                           *
 ****************************************************************************/

#include <chipcon/hal.h>
#include <chipcon/cc1010eb.h>


//----------------------------------------------------------------------------
//  word halReadPotSensor(...)
//
//  Description:
//      This function reads the CC1010EB potentiometer sensor from AD0 pin.
//  Arguments:
//      void
//  Return value:
//      word
//          10 bit sample from AD0
//----------------------------------------------------------------------------
word halReadPotSensor(void) {
    word val;

    // Configure ADC for potentiometer reading
    halConfigADC(ADC_MODE_SINGLE | ADC_REFERENCE_VDD, CC1010EB_CLKFREQ, 0);
    ADC_POWER(TRUE);

    // Wait for ADC to stabilize
    halWait(1, CC1010EB_CLKFREQ);

    // Select potentiometer input and read sample
    ADC_SELECT_INPUT(ADC_INPUT_AD0);
    ADC_SAMPLE_SINGLE();
    val = ADC_GET_SAMPLE_10BIT();

    return val;
}    
